using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using HPS;

namespace hps_cs_samples
{
    public partial class Sample
    {
        public void Exchange_Publish(WindowKey wk, Canvas canvas, ref View view, Model model, SegmentKey modelKey)
        {
            try
            {
            /* Sample: Exchange_Publish
                 * This sample will show you how to:
                 * - Load an Exchange file and check for IO exceptions
                 * - Output a 3D PDF file through Publish and check for IO exceptions
                 * 
                 * Read more about the topics covered in this sample in our Programming Guide 
                 * - Integration with HOOPS Exchange
                 * - Integration with HOOPS Publish */

            HPS.TextKit textKit = new HPS.TextKit();
            textKit.SetSize(0.02f, Text.SizeUnits.SubscreenRelative);
            HPS.Point[] point = { new HPS.Point(-0.95f, -0.70f, 10), new HPS.Point(0.95f, -0.70f, 10), new HPS.Point(0, -1, 10) };
            textKit.SetRegion(point, Text.RegionAlignment.Center, Text.RegionFitting.Left, false, false, true);
            textKit.SetBackground(true, "rounded box");
            textKit.SetAlignment(HPS.Text.Alignment.BottomLeft);
            HPS.MaterialMappingKit matMapKit = new HPS.MaterialMappingKit();
            matMapKit.SetFaceColor(new RGBAColor(1, 1, 1));
            matMapKit.SetEdgeColor(new RGBAColor(0, 0, 0));


            /*The next block of code show you how to set the Camera properties to something more familiar
             * 1. Set the camera to it's default properties
             * 2. Set the world handedness to "Left" for world positions akin to the Cartesian coordinates */

            view.GetSegmentKey().SetCamera(CameraKit.GetDefault());
            view.GetSegmentKey().GetDrawingAttributeControl().SetWorldHandedness(Drawing.Handedness.Left);

            /* This next block of code shows you how to load an Exchange file
             * 1. Create an ImportOptionsKit and set the segment in which you want the file to be loaded
             * 2. Use the Exchange.File.Import method to load the desired file, with the options you specified
             * 3. Since this operation happens on a separate thread, wait for it to complete before proceeding 
             * 4. Use a try-catch block to capture any Stream IO exceptions 
             * 5. Obtain an Exchange.CADModel object through the Exchange.ImportNotifier
             * 6. Obtain the new View from the CADModel object. Attach the new view to the Canvas and delete the old View
             *		You will need to re-insert geometry that was in the old View, such as lights.
             *		You will also need to re-apply any operators which were active on the old view.*/
            HPS.Exchange.ImportNotifier exchangeNotifier = new HPS.Exchange.ImportNotifier();
            try
            {
                string file_in = INPUT_FOLDER + "/Crank-Instructions.prc";
                exchangeNotifier = HPS.Exchange.File.Import(file_in, new HPS.Exchange.ImportOptionsKit());
                exchangeNotifier.Wait();

                HPS.CADModel modelFile = exchangeNotifier.GetCADModel();
                view.Delete();
                view = modelFile.ActivateDefaultCapture();
                canvas.AttachViewAsLayout(view);

                modelKey.InsertDistantLight(new HPS.Vector(1, 1, 0));
                view.FitWorld();
            }

            catch (HPS.IOException)
            {
                HPS.SegmentKey catchKey = view.GetSegmentKey().Down("", true);
                string error_message;
                error_message = "Exchange Import Unsuccessful";

                view.GetSegmentKey().GetVisibilityControl().SetEdges(true);
                view.GetSegmentKey().SetMaterialMapping(matMapKit);

                textKit.SetText(error_message);
                catchKey.InsertText(textKit);
                view.FitWorld();
                view.Update();
            }

            HPS.SegmentKey results_segment = view.GetSegmentKey().Down("", true);
            results_segment.GetVisibilityControl().SetEdges(true);
            view.GetSegmentKey().SetMaterialMapping(matMapKit);

            /* This next block of code shows you how to output a 3D PDF file
             * 1. Create SprocketPath object, by passing the keys to the canvas, layout, view and model to it
             * 2. Use the Publish.File.Export method to output a 3D PDF file */
            if (exchangeNotifier.Status() == HPS.IOResult.Success)
            {
                bool publishExportSuccess = true;

                try
                {
                    string file_out = OUTPUT_FOLDER + "/exchange_publish_sample.pdf";
                    HPS.SprocketPath sprkPath = new HPS.SprocketPath(canvas, canvas.GetAttachedLayout(),
                        canvas.GetAttachedLayout().GetAttachedView(),
                        canvas.GetAttachedLayout().GetAttachedView().GetAttachedModel());
                    HPS.KeyPath[] paths = { sprkPath.GetKeyPath() };
                    HPS.Publish.File.ExportPDF(paths, file_out, new HPS.Publish.ExportOptionsKit());
                }

                catch (HPS.IOException)
                {
                    string error_message;
                    publishExportSuccess = false;
                    error_message = "Publish Export Unsuccessful";
                    textKit.SetText(error_message);
                    view.GetAttachedModel().GetSegmentKey().InsertText(textKit);
                    view.FitWorld();
                    view.Update();
                }

                if (publishExportSuccess)
                {
                    string success_message = "Import and Export completed successfully\n" +
                        "You can find the 3D PDF here:\n" +
                        OUTPUT_FOLDER + "/exchange_publish_sample.pdf";

                    int index = success_message.IndexOf(@"/");
                    success_message = success_message.Substring(0, index) + "\n" + success_message.Substring(index, success_message.Length - index);

                    textKit.SetText(success_message);
                    results_segment.InsertText(textKit);
                    view.GetSegmentKey().GetCameraControl().SetTarget(new Point(-250, -100, 0));
                    view.Update();
                }

            }
            }
            catch (IOException e)
            {
                System.Console.WriteLine(e.Message);
            }
        }
    }
}
